#ifndef fft_cpp
#define fft_cpp

#pragma warn -sig
#pragma warn -csu

#include "fft.h"
#include <math.h>

Fft::Fft( int Points ) : _Points( Points ),
					_aBitRev( _Points ), pi( M_PI ), x( _Points )
{
	_sqrtPoints = sqrt( (double) _Points ) ;

	_logPoints = 0 ;
	Points-- ;
	while( Points != 0 )
	{
		Points >>= 1 ;
		_logPoints++ ;
	} ;

	_W.resize( _logPoints + 1 ) ;

	int _2_l = 2 ;
   double re, im ;

	for( int l=1 ; l < _W.limit() ; l++ )
	{
		_W[l].resize( _Points ) ;

		for( int i=0; i < _W[l].limit() ; i++ )
		{
			re =  cos( 2.0 * pi * i / _2_l ) ;
			im = -sin( 2.0 * pi * i / _2_l ) ;
			_W[l][i] = complex( re, im ) ;
		} ;
		_2_l <<= 1 ;
	} ;

	int rev = 0 ;
	int halfPoints = _Points >> 1 ;
   int mask ;

	for( int j=0 ; j < _Points - 1 ; j++ )
	{
		x[j] = complex( 0, 0 ) ;

		_aBitRev[j] = rev ;
		mask = halfPoints ;

		while( rev >= mask )
		{
			rev -= mask ;
			mask >>= 1 ;
		} ;
		rev += mask ;
	} ;
	_aBitRev[ _Points-1 ] = _Points - 1 ;
} ;

svector<complex>& Fft::Transform( svector<complex>& vc )
{
	int i ;

	if( vc.limit() != _Points )
		throw WrongNumberOfPoints() ;

	for( i=0; i<_Points ; i++ )
		x[ _aBitRev[i] ] = vc[i] ;

	_Transform() ;

	vc = x ;

	return vc ;
} ;

svector<double>& Fft::Transform( svector<double>& vc ) //, bool resize )
{
	int i ;

	if( vc.limit() != _Points )
		throw WrongNumberOfPoints() ;

	for( i=0; i<_Points ; i++ )
		x[ _aBitRev[i] ] = complex( vc[i], 0 ) ;

	_Transform() ;

//	if( resize )
//	{
		vc.resize( _Points / 2 ) ;
		for( i=0; i<vc.limit() ; i++ )
			vc[i] = abs( x[1+i] ) / _Points ;
//	}
//	else
//	for( i=0; i<_Points ; i++ )
//		vc[i] = abs( x[i] ) ;

	return vc ;
} ;

/*svector<int>& Fft::Transform( svector<int>& vc, bool resize )
{
	int i ;

	if( vc.limit() != _Points )
		throw WrongNumberOfPoints() ;

	for( i=0; i<_Points ; i++ )
		x[ _aBitRev[i] ] = complex( vc[i], 0 ) ;

	_Transform() ;

	if( resize )
	{
		vc.resize( _Points / 2 ) ;
		for( i=0; i<vc.limit() ; i++ )
			vc[i] = round( abs( x[1+i] ) ) ;
	}
	else
		for( i=0; i<_Points ; i++ )
			vc[i] = round( abs( x[i] ) ) ;

	return vc ;
} ;

svector<char>& Fft::Transform( svector<char>& vc, bool resize )
{
	int i ;

	if( vc.limit() != _Points )
		throw WrongNumberOfPoints() ;

	for( i=0; i<_Points ; i++ )
		x[ _aBitRev[i] ] = complex( vc[i], 0 ) ;

	_Transform() ;

	if( resize )
	{
		vc.resize( _Points / 2 ) ;
		for( i=0; i<vc.limit() ; i++ )
			vc[i] = round( abs( x[1+i] ) ) ;
	}
	else
		for( i=0; i<_Points ; i++ )
			vc[i] = round( abs( x[i] ) ) ;

	return vc ;
} ;
*/

void Fft::_Transform()
{
	int step = 1 ;
	int increm ;
	int i, j ;
	complex u, t ;


	for( int level=1; level < _W.limit(); level++ )
	{
		increm = step*2 ;
		for( j=0; j<step; j++ )
		{
			u = _W [level] [j] ;
			for( i=j; i<_Points ; i += increm )
			{
				t = u ;
				t *= x [i+step] ;
				x [i+step] = x[i] ;
				x [i+step] -= t ;
				x [i] += t ;
			} ;
		} ;
		step <<= 1 ;
	} ;
} ;

#endif
