#ifndef _SVECTOR_H
#define _SVECTOR_H

#include <limits.h>
#include <assert.h>

//#include "bool.h"

template <class T>
class svector
{
public:

	svector() : data(0), lim(0) { } ;
	svector( unsigned sz ) : data( new T[sz] ), lim(sz) { assert( data != 0 ) ; } ;
	svector( const svector<T> _FAR & );
	~svector() { delete [] data; } ;

	const svector<T> _FAR & operator = ( const svector<T> _FAR & );

	T _FAR & operator [] ( unsigned index ) const
	{
		assert( lim > 0 && data != 0 && index < lim );
		return data[index];
	} ;

	unsigned limit() const { return lim; } ;
	void resize( unsigned ) ;
protected:
	T _FAR * data;
	unsigned lim;
} ;

template <class T>
svector<T>::svector( const svector<T> _FAR & v ) : data( new T[v.lim] ) , lim(v.lim)
{
	assert( lim == 0 || (data != 0 && v.data != 0) );
	for( unsigned i = 0; i < lim; i++ )
		data[i] = v.data[i] ;
} ;

template <class T>
const svector<T> _FAR & svector<T>::operator = ( const svector<T> _FAR & v )
{
	if( data != v.data )
	{
		delete [] data;
		data = new T[v.lim];
		assert( data != 0 );
		lim = v.lim;
		for( unsigned i = 0; i < lim; i++ )
			data[i] = v.data[i];
	}
	return *this;
} ;

template <class T>
void svector<T>::resize( unsigned newSz )
{
	if( newSz == lim ) return ;

	T _FAR *temp = new T[ newSz ] ;
   assert( temp != 0 ) ;

	unsigned last = min( newSz, lim );
	for( unsigned i = 0 ; i < last ; i++ )
		temp[i] = data[i] ;

	delete [] data;
	data = temp;
	lim = newSz;
} ;

template <class T>
svector<T>& VectorNormalize( svector<T>& vc, double max_ )
{
	double temp = 0.0 ;
	unsigned int i ;

	for( i=0; i<vc.limit() ; i++ )
		temp = max( temp, abs( vc[i] ) ) ;

	if( temp == 0.0 )
		return vc ;

	max_ /= temp ;

	for( i=0; i<vc.limit() ; i++ )
		vc[i] *= max_ ;

	return vc ;
} ;

template <class T>
svector< svector<T> >& MatrixNormalize( svector< svector<T> >& vc, double max_ )
{
	double temp = 0.0 ;
	int i, j ;

	for( i=0; i<vc.limit() ; i++ )
		for( j=0 ; j<vc[i].limit() ; j++ )
      	temp = max( temp, abs( vc[i][j] ) ) ;

	if( temp == 0.0 )
		return vc ;

	max_ /= temp ;

	for( i=0; i<vc.limit() ; i++ )
		for( j=0; j<vc[i].limit() ; j++ )
			vc[i][j] *= max_ ;

	return vc ;
} ;

#endif
