#ifndef wavein_cpp
#define wavein_cpp

#pragma warn -sig
#include "wavein.h"

Recorder::Recorder( WORD cSamples, DWORD cSamplesPerSec, WORD nChannels, WORD bits )
:	_cSamples( cSamples ),
	_cSamplesPerSec( cSamplesPerSec ),
	_nChannels( nChannels ),
	_bits( bits ),
	_iBuf(0),
	_cbBuf( cSamples * nChannels * bits / 8 )
{
	_dataPool = new char[ _cbBuf*NUM_BUF ] ;
	assert( _dataPool != NULL ) ;
} ;

Recorder::~Recorder()
{
	Stop() ;
	delete [] _dataPool ;
} ;

bool Recorder::Start( HWND hwnd )
{
	LPSTR errorText ;

	WaveFormat format( _cSamplesPerSec, _nChannels, _bits ) ;

	_wave.Open( hwnd, 0, format ) ;

	if( !_wave.ok() )
	{
		if( _wave.isInUse() )
			errorText = "Outro aplicativo est utilizando o dispositivo de entrada." ;
		else
			errorText = _wave.queryError() ;

		MessageBox( hwnd, errorText, _wave.queryTitle(), MB_ICONSTOP ) ;
		return false ;
	} ;

	for( int i=0; i<NUM_BUF - 1; i++ )
	{
		_header[i].lpData = &_dataPool[ i*_cbBuf ] ;
		_header[i].dwBufferLength = _cbBuf ;
		_header[i].dwFlags = 0 ;
		_header[i].dwLoops = 0 ;

		_wave.Prepare( &_header[i] ) ;
		_wave.Send( &_header[i] ) ;
	} ;

	_iBuf = 0 ;
	_wave.Start() ;
	return true ;
} ;

void Recorder::Stop()
{
//	_wave.Stop() ;
	_wave.Reset() ;
	_wave.Close() ;
} ;

bool Recorder::BufferDone()
{
	if( !_wave.ok() ) return false ;

	assert( isBufferDone() ) ;

	_wave.UnPrepare( &_header[ _iBuf ] ) ;

	int prevBuf = _iBuf - 1 ;
	if( prevBuf < 0 ) prevBuf = NUM_BUF - 1 ;

	_iBuf++ ;
	if( _iBuf == NUM_BUF ) _iBuf = 0 ;

	_header[prevBuf].lpData = &_dataPool[ prevBuf*_cbBuf ] ;
	_header[prevBuf].dwBufferLength = _cbBuf ;
	_header[prevBuf].dwFlags = 0 ;
	_header[prevBuf].dwLoops = 0 ;

	_wave.Prepare( &_header[prevBuf] ) ;
	_wave.Send( &_header[prevBuf] ) ;

	return true ;
} ;

bool Recorder::FillVector( svector<double>& vc )
{
   #pragma warn -csu
	if( !_wave.ok() ) return false ;

	assert( isBufferDone() ) ;

	int i ;
	WaveHeader* hdr = &_header[_iBuf] ;
	LPSTR data = hdr->lpData ;

	vc.resize( hdr->dwBytesRecorded / (_nChannels*_bits/8) ) ;

	if( _bits == 8 && _nChannels == 1 )
	{
		for( i=0 ; i<vc.limit() ; i++ )
			vc[i] = (((unsigned char)data[i]-128)) ;
	}
	else if( _bits == 16 && _nChannels == 1 )
	{
		for( i=0 ; i<vc.limit() ; i++ )
			vc[i] = (((short*)data)[i]) / 64 ;
	}
	else return false ;
	return true ;
   #pragma warn +csu
   } ;

#endif
